

#include "FastLED.h"
#include <string.h>

// How many leds in your strip?
#define NUM_LEDS 50
// For led chips like Neopixels, which have a data line, ground, and power, you just
// need to define DATA_PIN.  For led chipsets that are SPI based (four wires - data, clock,
// ground, and power), like the LPD8806, define both DATA_PIN and CLOCK_PIN
#define DATA_PIN 2


// Define the array of leds
CRGB leds[NUM_LEDS];

CRGB twinkle_colours[4];
#define ETL_BLUE 0x54B7C6

CHSV blueHSV=CHSV(133,147,198);

enum patterns {   
  NOTHING,
  SWEEP,   //LED band travels with end wrap
  BOUNCE,  //LED band reverses direction at ends
  TWINKLE  //random LED(s) get turned on every frame
};


enum postProcesses {
  NOWT,  //do nothing
  FADE,     //fade to a colour
  SHIFT    //move all pixels
};

//define various timings (in ms)
int fadeUpTime = 5000; //time from power up to background colour
int prePatternPause = 4000; //time from fade up to running pattern 
patterns pattern = NOTHING;
postProcesses postProcess = NOWT;

float fadeRate = 5.0;

void setup() { 
  FastLED.addLeds<WS2811,DATA_PIN,RGB>(leds, NUM_LEDS);
  FastLED.setMaxPowerInVoltsAndMilliamps(5,1000); 
  Serial.begin(19200);
  Serial.setTimeout(100);
  
  fill_solid(&(leds[0]),NUM_LEDS, CRGB(0,0,0)); 
  pattern = TWINKLE;  
  set_colours(3);
  
}


void set_colours(byte colour) {
  switch(colour) {
    case 1: { //red white blue green
      twinkle_colours[0] = CRGB(255,255,255);
      twinkle_colours[1] = CRGB(255,0,0);
      twinkle_colours[2] = CRGB(0,255,0);
      twinkle_colours[3] = CRGB(0,0,255); 
    } break;

     case 2: { //Purple green red green
      twinkle_colours[0] = CRGB::DarkViolet;
      twinkle_colours[1] = CRGB::ForestGreen;
      twinkle_colours[2] = CRGB(255,0,0);
      twinkle_colours[3] = CRGB(255,0,0); 
    } break;

      case 3: { //green gold blue green
      twinkle_colours[0] = CRGB::ForestGreen;
      twinkle_colours[1] = CRGB::LawnGreen;
      twinkle_colours[2] = CRGB(0,0,200);
      twinkle_colours[3] = CRGB::Gold ; 
    } break;

    case 4: { //purple pink yellow blue
      twinkle_colours[0] = CRGB::Yellow;
      twinkle_colours[1] = CRGB::Fuchsia;
      twinkle_colours[2] = CRGB::RoyalBlue;
      twinkle_colours[3] = CRGB::Purple; 
    } break;
  
   case 5: { // just greens
      twinkle_colours[0] = CRGB::ForestGreen;
      twinkle_colours[1] = CRGB::LawnGreen;
      twinkle_colours[2] = CRGB(0,255,0);
      twinkle_colours[3] = CRGB::Chartreuse;
    } break;
   case 6: { // just red
      twinkle_colours[0] = CRGB(255,0,0);
      twinkle_colours[1] = CRGB(255,0,0);
      twinkle_colours[2] = CRGB(255,0,0);
      twinkle_colours[3] = CRGB(255,0,0);
    } break;
    case 7: { // just Blue
      twinkle_colours[0] = CRGB(0,0,255);
      twinkle_colours[1] = CRGB(0,0,255);
      twinkle_colours[2] = CRGB(0,0,255);
      twinkle_colours[3] = CRGB(0,0,255);
    } break;
  
}
}

void loop() { 
  static int sweepPlace = 0;
  static CHSV backGndHSV = CHSV(0,0,0);
  static int sweepFrameTime = 1;
  static int sweepPause = 0;
  static int frameDelay = 50;
  static CRGB sweepBandColour = CRGB::White;
  static int sweepBandWidth = 2;
  static int numTwinkles = 2;
  static unsigned long frameTime = 0;
  static unsigned long lastMillis = 0;
  static unsigned int twinkleMillis = 0;
  static int twinkleTime = 100;
  if(lastMillis != 0 ) {
    frameTime = millis() - lastMillis;
  }
  lastMillis = millis();
  doFade(backGndHSV, fadeRate);
  
  switch(pattern) {
    case NOTHING:  
       delay(1); //just a hunch
       break ;
       
  case TWINKLE : {
   //  fill_solid(&(leds[0]),NUM_LEDS, backGndHSV);
     twinkleMillis += frameTime;
     if( twinkleMillis >= twinkleTime ) {
       twinkleMillis = 0;
       for (int i = 0;i<numTwinkles;i++) {
          leds[random(NUM_LEDS)] = twinkle_colours[random(4)];
       }
     }  
     break; 
  }
  
} //end of per frame process switch
  
  FastLED.show(); 
  
  if (Serial.available() >= 3) {
   char inp[128]; 
   byte charsRead = 0;
   charsRead = Serial.readBytesUntil('/n',inp,128);
   // quick validation all commands are of the form XY:.... so check third char is a colon
   if( inp[2] == ':' ) {
     // add null termination to help out subsequent parsers
     inp[charsRead] = 0;
     if (inp[0]=='B' && inp[1] == 'G' ) { //set background colour
         backGndHSV = parseHSV(&inp[3]);
         sendack();
     }
     
     else if (inp[0]=='F' && inp[1] == 'S' ) { //report back frame size (NUM_LEDS)
         Serial.println(NUM_LEDS);
         sendack();
     }
     else if (inp[0]=='F' && inp[1] == 'H' ) { //accept a frame of HSV values, separate LED valuse by semicolon thus:  H:xxx,S:xxx,V:xxx;H:xxx;S:xxx:V:xxx;... in pixel order
         setFrameHSV(&inp[3]);
         pattern = NOTHING;
         sendack();
     }
     else if (inp[0]=='F' && inp[1] == 'R' ) { //accept a frame of RGB values, separate LED valuse by semicolon thus:  R:xxx,G:xxx,B:xxx;R:xxx;G:xxx:B:xxx;... in pixel order
         setFrameRGB(&inp[3]);
         pattern = NOTHING;
         sendack();
     }
     else if (inp[0]=='B' &&inp[1] == 'R') { // set brightness
         int brt = atoi(&inp[3]);
         FastLED.setBrightness(brt);
         sendack();
     }
     else if (inp[0]=='S' &&inp[1] == 'C') { // set twinkle colours
         int pat = atoi(&inp[3]);
         set_colours(pat);
         sendack();
     }
    }
  } 
  delay(frameDelay);  
} //end of loop()

void sendack() 
{
  Serial.println("AK:");
}


void setFrameHSV(char *instring) {
  char *token, *progress;
  int led = 0;
  token = strtok_r(instring,";",&progress);
  while (token != NULL) {
    leds[led]=parseHSV(token);
    token = strtok_r(NULL,";",&progress);
    led ++ ;
  }  
}

void setFrameRGB(char *instring) {
  char *token, *progress;
  int led = 0;
  Serial.print("RGB frame, input = ");
  Serial.println(instring);
  token = strtok_r(instring,";",&progress);
  while (token != NULL) {
    leds[led]=parseRGB(token);
    token = strtok_r(NULL,";",&progress);
    led ++ ; 
  }  
}


CHSV parseHSV(char *instring) 
{
  int H =0;
  int S =0;
  int V =0;
  char *token, *progress ;
  /*Serial.print("parseHSV input= ");
  Serial.write(instring);
  Serial.println();  */
  token = strtok_r(instring,",",&progress);
  for (int i=0;i<=2;i++ ) {
    if(token[0] == 'H' ) {
      H = atoi(token + 2);
    }
    else if (token[0] == 'S') {
      S = atoi(token + 2);
    }
    else if (token[0] == 'V') {
      V = atoi(token + 2);
    }  
    token = strtok_r(NULL,",",&progress);
  }  /*
  Serial.print("parseHSV-> H=");
  Serial.print(H);
  Serial.print(", S=");
  Serial.print(S);
  Serial.print(", V=");
  Serial.println(V);
   */
  return CHSV(H,S,V);  
  //could probably do with some error checking here one day.
}

CRGB parseRGB(char *instring) 
{
  byte R =0;
  byte G =0;
  byte B =0;
  char *token, *progress ;
  Serial.print("parseRGB input= ");
  Serial.write(instring);
  Serial.println();
  token = strtok_r(instring,",",&progress);
  for (int i=0;i<=2;i++ ) {
    if(token[0] == 'R' ) {
      R = atoi(token + 2);
    }
    else if (token[0] == 'G') {
      G = atoi(token + 2);
    }
    else if (token[0] == 'B') {
      B = atoi(token + 2);
    }  
    token = strtok_r(NULL,",",&progress);
  }  
 /* Serial.print("parseRGB-> R=");
  Serial.print(R);
  Serial.print(", G=");
  Serial.print(G);
  Serial.print(", B=");
  Serial.println(B);
 */  
  return CRGB(R,G,B);  
  //could probably do with some error checking here one day.
}

void doFade(CHSV targetColour,float rate)// fade array towards RGB value given ... for perisitence effects.
{
  byte r = CRGB(targetColour).r;
  byte g = CRGB(targetColour).g;
  byte b = CRGB(targetColour).b;
  for(int i = 0; i< NUM_LEDS;i++) 
   {
     if(leds[i].r != r) 
      {
        float diff = leds[i].r - r;
        diff = diff / rate;
       // if (abs(diff) < 1) diff = (diff < 0) ? -1 : 1;  
        if( diff >0) diff = max(diff,1);
        else diff =min(diff,-1);  
        leds[i].r = leds[i].r - diff;
      } 
      
    if(leds[i].g != g) 
      {
        float diff = leds[i].g - g;
        diff = diff / rate;
         //if (abs(diff) < 1) diff = (diff < 0) ? -1 : 1;  
        if( diff >0) diff = max(diff,1);
        else diff =min(diff,-1);   
        leds[i].g = leds[i].g - diff;
      } 
     
    if(leds[i].b != b )
      {
        float diff = leds[i].b - b;
        diff = diff / rate;
        // if (abs(diff) < 1) diff = (diff < 0) ? -1 : 1; 
        if( diff >0) diff = max(diff,1);
        else diff =min(diff,-1);      
        leds[i].b = leds[i].b - diff;
      }  
   }
}


 
